# Generated by roxygen2: do not edit by hand

S3method(.cal_table_breaks,data.frame)
S3method(.cal_table_breaks,tune_results)
S3method(.cal_table_logistic,data.frame)
S3method(.cal_table_logistic,tune_results)
S3method(.cal_table_windowed,data.frame)
S3method(.cal_table_windowed,tune_results)
S3method(any_equivocal,class_pred)
S3method(any_equivocal,default)
S3method(as_class_pred,default)
S3method(as_class_pred,factor)
S3method(cal_apply,cal_object)
S3method(cal_apply,data.frame)
S3method(cal_apply,tune_results)
S3method(cal_estimate_beta,data.frame)
S3method(cal_estimate_beta,grouped_df)
S3method(cal_estimate_beta,tune_results)
S3method(cal_estimate_isotonic,data.frame)
S3method(cal_estimate_isotonic,grouped_df)
S3method(cal_estimate_isotonic,tune_results)
S3method(cal_estimate_isotonic_boot,data.frame)
S3method(cal_estimate_isotonic_boot,grouped_df)
S3method(cal_estimate_isotonic_boot,tune_results)
S3method(cal_estimate_linear,data.frame)
S3method(cal_estimate_linear,grouped_df)
S3method(cal_estimate_linear,tune_results)
S3method(cal_estimate_logistic,data.frame)
S3method(cal_estimate_logistic,grouped_df)
S3method(cal_estimate_logistic,tune_results)
S3method(cal_estimate_multinomial,data.frame)
S3method(cal_estimate_multinomial,grouped_df)
S3method(cal_estimate_multinomial,tune_results)
S3method(cal_plot_breaks,data.frame)
S3method(cal_plot_breaks,grouped_df)
S3method(cal_plot_breaks,tune_results)
S3method(cal_plot_logistic,data.frame)
S3method(cal_plot_logistic,grouped_df)
S3method(cal_plot_logistic,tune_results)
S3method(cal_plot_regression,data.frame)
S3method(cal_plot_regression,grouped_df)
S3method(cal_plot_regression,tune_results)
S3method(cal_plot_windowed,data.frame)
S3method(cal_plot_windowed,grouped_df)
S3method(cal_plot_windowed,tune_results)
S3method(cal_validate_beta,resample_results)
S3method(cal_validate_beta,rset)
S3method(cal_validate_beta,tune_results)
S3method(cal_validate_isotonic,resample_results)
S3method(cal_validate_isotonic,rset)
S3method(cal_validate_isotonic,tune_results)
S3method(cal_validate_isotonic_boot,resample_results)
S3method(cal_validate_isotonic_boot,rset)
S3method(cal_validate_isotonic_boot,tune_results)
S3method(cal_validate_linear,resample_results)
S3method(cal_validate_linear,rset)
S3method(cal_validate_logistic,resample_results)
S3method(cal_validate_logistic,rset)
S3method(cal_validate_logistic,tune_results)
S3method(cal_validate_multinomial,resample_results)
S3method(cal_validate_multinomial,rset)
S3method(cal_validate_multinomial,tune_results)
S3method(collect_metrics,cal_rset)
S3method(collect_predictions,cal_rset)
S3method(format,class_pred)
S3method(int_conformal_cv,default)
S3method(int_conformal_cv,resample_results)
S3method(int_conformal_cv,tune_results)
S3method(int_conformal_full,default)
S3method(int_conformal_full,workflow)
S3method(int_conformal_quantile,workflow)
S3method(int_conformal_split,default)
S3method(int_conformal_split,workflow)
S3method(is_equivocal,class_pred)
S3method(is_equivocal,default)
S3method(is_ordered,class_pred)
S3method(is_ordered,default)
S3method(levels,class_pred)
S3method(obj_print_data,class_pred)
S3method(obj_print_footer,class_pred)
S3method(obj_print_header,class_pred)
S3method(predict,int_conformal_cv)
S3method(predict,int_conformal_full)
S3method(predict,int_conformal_quantile)
S3method(predict,int_conformal_split)
S3method(print,cal_binary)
S3method(print,cal_estimate_isotonic)
S3method(print,cal_estimate_linear)
S3method(print,cal_multi)
S3method(print,cal_regression)
S3method(print,int_conformal_cv)
S3method(print,int_conformal_full)
S3method(print,int_conformal_quantile)
S3method(print,int_conformal_split)
S3method(reportable_rate,class_pred)
S3method(reportable_rate,default)
S3method(required_pkgs,cal_estimate_beta)
S3method(required_pkgs,cal_estimate_linear_spline)
S3method(required_pkgs,cal_estimate_logistic_spline)
S3method(required_pkgs,cal_estimate_multinomial)
S3method(required_pkgs,cal_object)
S3method(threshold_perf,data.frame)
S3method(type_sum,cal_object)
S3method(vec_cast,character.class_pred)
S3method(vec_cast,class_pred.character)
S3method(vec_cast,class_pred.class_pred)
S3method(vec_cast,class_pred.factor)
S3method(vec_cast,class_pred.ordered)
S3method(vec_cast,factor.class_pred)
S3method(vec_cast,ordered.class_pred)
S3method(vec_proxy_compare,class_pred)
S3method(vec_proxy_equal,class_pred)
S3method(vec_ptype2,character.class_pred)
S3method(vec_ptype2,class_pred.character)
S3method(vec_ptype2,class_pred.class_pred)
S3method(vec_ptype2,class_pred.factor)
S3method(vec_ptype2,factor.class_pred)
S3method(vec_ptype_abbr,class_pred)
S3method(which_equivocal,class_pred)
S3method(which_equivocal,default)
export(.cal_table_breaks)
export(.cal_table_logistic)
export(.cal_table_windowed)
export(any_equivocal)
export(append_class_pred)
export(as.factor)
export(as.ordered)
export(as_class_pred)
export(augment)
export(cal_apply)
export(cal_estimate_beta)
export(cal_estimate_isotonic)
export(cal_estimate_isotonic_boot)
export(cal_estimate_linear)
export(cal_estimate_logistic)
export(cal_estimate_multinomial)
export(cal_plot_breaks)
export(cal_plot_logistic)
export(cal_plot_regression)
export(cal_plot_windowed)
export(cal_validate_beta)
export(cal_validate_isotonic)
export(cal_validate_isotonic_boot)
export(cal_validate_linear)
export(cal_validate_logistic)
export(cal_validate_multinomial)
export(class_pred)
export(collect_metrics)
export(collect_predictions)
export(control_conformal_full)
export(fit)
export(int_conformal_cv)
export(int_conformal_full)
export(int_conformal_quantile)
export(int_conformal_split)
export(is_class_pred)
export(is_equivocal)
export(make_class_pred)
export(make_two_class_pred)
export(reportable_rate)
export(required_pkgs)
export(threshold_perf)
export(which_equivocal)
import(ggplot2)
import(rlang)
import(vctrs)
importFrom(dplyr,"%>%")
importFrom(generics,as.factor)
importFrom(generics,as.ordered)
importFrom(generics,augment)
importFrom(generics,fit)
importFrom(generics,required_pkgs)
importFrom(pillar,type_sum)
importFrom(purrr,map)
importFrom(stats,as.stepfun)
importFrom(stats,binomial)
importFrom(stats,glm)
importFrom(stats,isoreg)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,prop.test)
importFrom(stats,qnorm)
importFrom(tune,collect_metrics)
importFrom(tune,collect_predictions)
importFrom(utils,head)
importFrom(yardstick,j_index)
importFrom(yardstick,sens)
importFrom(yardstick,spec)
