# split conformal intervals

    No known 'int_conformal_split' methods for this type of object.

---

    The following required columns are missing: 'outcome'.

---

    The following required columns are missing: 'predictor_01'.

---

    `...` must be empty.
    x Problematic argument:
    * level = 0.1

---

    `...` must be empty.
    x Problematic argument:
    * potato = 3

---

    Code
      lm_int
    Output
      Split Conformal inference
      preprocessor: formula 
      model: linear_reg (engine = lm) 
      calibration set size: 100 
      
      Use `predict(object, new_data, level)` to compute prediction intervals

