# slicing

    Code
      manual_creation_eq[1:6]
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Location 6 doesn't exist.
      i There are only 5 elements.

