# Logistic validation with data frame input

    There are no saved prediction columns to collect.

# Isotonic classification validation with `fit_resamples`

    'truth' is automaticaly set when this type of object is used.

# validation functions error with tune_results input

    This function can only be used with an <rset> object or the results of `tune::fit_resamples()` with a .predictions column.
    i Not an <tune_results> object.

---

    This function can only be used with an <rset> object or the results of `tune::fit_resamples()` with a .predictions column.
    i Not an <tune_results> object.

---

    This function can only be used with an <rset> object or the results of `tune::fit_resamples()` with a .predictions column.
    i Not an <tune_results> object.

---

    no applicable method for 'cal_validate_linear' applied to an object of class "c('tune_results', 'tbl_df', 'tbl', 'data.frame')"

---

    This function can only be used with an <rset> object or the results of `tune::fit_resamples()` with a .predictions column.
    i Not an <tune_results> object.

---

    This function can only be used with an <rset> object or the results of `tune::fit_resamples()` with a .predictions column.
    i Not an <tune_results> object.

