# fails with different length `...`

    All vectors passed to `...` must be of the same length.

# fails with different type `...`

    x The index supplied to `...` are not numeric:
    i 2

# fails with different length `...` VS levels

    `levels` must be a character vector with the same length as the number of vectors passed to `...`.

# validates type of `levels` (#42)

    `levels` must be a character vector of length 2.

---

    `levels` must be a character vector with the same length as the number of vectors passed to `...`.

