# Binary breaks functions work with group argument

    x `.by` cannot select more than one column.
    i The following 2 columns were selected:
    i group1 and group2

# breaks plot function errors - grouped_df

    x This function does not work with grouped data frames.
    i Apply `dplyr::ungroup()` and use the `.by` argument.

# Binary logistic functions work with group argument

    x `.by` cannot select more than one column.
    i The following 2 columns were selected:
    i group1 and group2

# logistic plot function errors - grouped_df

    x This function does not work with grouped data frames.
    i Apply `dplyr::ungroup()` and use the `.by` argument.

# windowed plot function errors - grouped_df

    x This function does not work with grouped data frames.
    i Apply `dplyr::ungroup()` and use the `.by` argument.

# Event level handling works

    Invalid event_level entry: invalid. Valid entries are 'first', 'second', or 'auto'

# regression plot function errors - grouped_df

    x This function does not work with grouped data frames.
    i Apply `dplyr::ungroup()` and use the `.by` argument.

