# split conformal quantile intervals

    no applicable method for 'int_conformal_quantile' applied to an object of class "lm"

---

    The following required columns are missing: 'predictor_01'.

---

    The following required columns are missing: 'predictor_01'.

---

    `...` must be empty.
    x Problematic argument:
    * level = 0.9

---

    Code
      lm_int
    Output
      Split Conformal inference via Quantile Regression
      preprocessor: formula 
      model: linear_reg (engine = lm) 
      calibration set size: 100 
      confidence level: 0.9 
      
      Use `predict(object, new_data)` to compute prediction intervals

